 /* Copyright 2010-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPS_PLUS_CONVERSION_FUNCTIONS_H
#define NV_NPPS_PLUS_CONVERSION_FUNCTIONS_H
 
/**
 * \file nppsPlus_conversion_functions.h
 * NPP Signal Processing Functionality.
 */

/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 *  \page signal_conversion_functions Conversion Functions
 *  @defgroup signal_conversion_functions Conversion Functions
 *  @ingroup npps
 * Functions that provide conversion and threshold operations
 * @{
 *
 */

/** 
 * \section signal_convert Convert
 * @defgroup signal_convert Convert
 * The set of conversion operations available in the library
 * @{
 *
 */

/** @name Convert
 * Routines for converting the sample-data type of signals.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32U, NPP_32S, NPP_64S, NPP_32F, NPP_64F. 
 * \param eSrcType source \ref signal_data_type.
 * \param eDstType destination \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pDst \ref destination_signal_pointer.
 * \param eRoundMode \ref rounding_mode_parameter.
 * \param nScaleFactor \ref integer_result_scaling, 0 for no scaling.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Convert Up
 */ 
NPP_EXTERN_C
NppStatus 
nppsConvert_Up_Ctx(NppDataType eSrcType, NppDataType eDstType, 
                   const void * pSrc, void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Convert 16s32f Sfs
 */ 
NPP_EXTERN_C
NppStatus 
nppsConvert_16s32f_Sfs_Ctx(const Npp16s * pSrc, Npp32f * pDst, size_t nLength, 
                           int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Convert 16s64f Sfs
 */ 
NPP_EXTERN_C
NppStatus 
nppsConvert_16s64f_Sfs_Ctx(const Npp16s * pSrc, Npp64f * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Convert 32s64f Sfs
 */ 
NPP_EXTERN_C
NppStatus 
nppsConvert_32s64f_Sfs_Ctx(const Npp32s * pSrc, Npp64f * pDst, size_t nLength, int nScaleFactor, NppStreamContext nppStreamCtx);

/** 
 * Convert Down
 */ 
NPP_EXTERN_C
NppStatus 
nppsConvert_Down_Ctx(NppDataType eSrcType, NppDataType eDstType, 
                     const void * pSrc, void * pDst, size_t nLength, NppStreamContext nppStreamCtx);

/** 
 * Convert Down Sfs
 */ 
NPP_EXTERN_C
NppStatus 
nppsConvert_Down_Sfs_Ctx(NppDataType eSrcType, NppDataType eDstType, 
                         const void * pSrc, void * pDst, 
                         NppRoundMode eRoundMode, int nScaleFactor, 
                         size_t nLength, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} end of Convert */

/** @} signal_convert */

/** 
 * \section signal_threshold Threshold
 *  @defgroup signal_threshold Threshold
 * The set of threshold operations available in the library.
 * @{
 *
 */

/** @name Threshold Functions
 * Performs the threshold operation on the samples of a signal by limiting the sample values by a specified constant value.
 *
 * @{
 *
 */

#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/** 
 * Signal threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, const void * pLevel, NppCmpOp nRelOp, NppStreamContext nppStreamCtx);

/** 
 * In place signed short signal threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param nRelOp NppCmpOp type of thresholding operation (NPP_CMP_LESS or NPP_CMP_GREATER only).
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_I_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, const void * pLevel, NppCmpOp nRelOp, NppStreamContext nppStreamCtx);

/** 
 * Signal NPP_CMP_LESS threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_LT_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, const void * pLevel, NppStreamContext nppStreamCtx);

/** 
 * In place signed short signal NPP_CMP_LESS threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_LT_I_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, const void * pLevel, NppStreamContext nppStreamCtx);

/** 
 * Signal NPP_CMP_GREATER threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_GT_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, const void * pLevel, NppStreamContext nppStreamCtx);

/** 
 * In place signed short signal NPP_CMP_GREATER threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_GT_I_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, const void * pLevel, NppStreamContext nppStreamCtx);

/** 
 * Signal NPP_CMP_LESS threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param pValue Host memory pointer to constant eSrcDstType value to replace source value when threshold test is true.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_LTVal_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, 
                        const void * pLevel, const void * pValue, NppStreamContext nppStreamCtx);

/** 
 * In place signed short signal NPP_CMP_LESS threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param pValue Host memory pointer to constant eSrcDstType value to replace source value when threshold test is true.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_LTVal_I_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, 
                          const void * pLevel, const void * pValue, NppStreamContext nppStreamCtx);

/** 
 * Signal NPP_CMP_GREATER threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrc \ref source_signal_pointer.
 * \param pDst \ref destination_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param pValue Host memory pointer to constant eSrcDstType value to replace source value when threshold test is true.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_GTVal_Ctx(NppDataType eSrcDstType, const void * pSrc, void * pDst, size_t nLength, 
                        const void * pLevel, const void * pValue, NppStreamContext nppStreamCtx);

/** 
 * In place floating point signal NPP_CMP_GREATER threshold with constant level.
 * Supported data types include NPP_8U, NPP_8S, NPP_16U, NPP_16S, NPP_32F, NPP_64F. 
 * \param eSrcDstType \ref signal_data_type.
 * \param pSrcDst \ref in_place_signal_pointer.
 * \param nLength \ref length_specification.
 * \param pLevel Host memory pointer to constant threshold eSrcDstType value to be used to limit each signal sample
 * \param pValue Host memory pointer to constant eSrcDstType value to replace source value when threshold test is true.
 * \param nppStreamCtx \ref application_managed_stream_context. 
 * \return \ref signal_data_error_codes, \ref length_error_codes.
 */
NPP_EXTERN_C
NppStatus 
nppsThreshold_GTVal_I_Ctx(NppDataType eSrcDstType, void * pSrcDst, size_t nLength, 
                          const void * pLevel, const void * pValue, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} end of Threshold */

/** @} signal_threshold */

/** @} signal_conversion_functions */

#endif /* NV_NPPS_PLUS_CONVERSION_FUNCTIONS_H */
