 /* Copyright 2010-2024 NVIDIA CORPORATION & AFFILIATES.  All rights reserved. 
  * 
  * NOTICE TO LICENSEE: 
  * 
  * The source code and/or documentation ("Licensed Deliverables") are 
  * subject to NVIDIA intellectual property rights under U.S. and 
  * international Copyright laws. 
  * 
  * The Licensed Deliverables contained herein are PROPRIETARY and 
  * CONFIDENTIAL to NVIDIA and are being provided under the terms and 
  * conditions of a form of NVIDIA software license agreement by and 
  * between NVIDIA and Licensee ("License Agreement") or electronically 
  * accepted by Licensee.  Notwithstanding any terms or conditions to 
  * the contrary in the License Agreement, reproduction or disclosure 
  * of the Licensed Deliverables to any third party without the express 
  * written consent of NVIDIA is prohibited. 
  * 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE 
  * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  THEY ARE 
  * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND. 
  * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED 
  * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, 
  * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE. 
  * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE 
  * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY 
  * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY 
  * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
  * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS 
  * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
  * OF THESE LICENSED DELIVERABLES. 
  * 
  * U.S. Government End Users.  These Licensed Deliverables are a 
  * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT 
  * 1995), consisting of "commercial computer software" and "commercial 
  * computer software documentation" as such terms are used in 48 
  * C.F.R. 12.212 (SEPT 1995) and are provided to the U.S. Government 
  * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and 
  * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all 
  * U.S. Government End Users acquire the Licensed Deliverables with 
  * only those rights set forth herein. 
  * 
  * Any use of the Licensed Deliverables in individual and commercial 
  * software must include, in the user documentation and internal 
  * comments to the code, the above Disclaimer and U.S. Government End 
  * Users Notice. 
  */ 
#ifndef NV_NPPS_PLUS_FILTERING_FUNCTIONS_H
#define NV_NPPS_PLUS_FILTERING_FUNCTIONS_H
 
/**
 * \file nppsPlus_filtering_functions.h
 * NPP Signal Processing Functionality.
 */

/**
 * Supports namespace builds on Linux
 */

#if (defined(_WIN32) || defined(_WIN64))
#define NPP_EXTERN_C
#else /* (defined(_WIN32) || defined(_WIN64)) */
#if !defined NPP_EXTERN_C
#define NPP_EXTERN_C  extern "C"
#endif
#endif /* (defined(_WIN32) || defined(_WIN64)) */

#include "nppdefs.h"

/** 
 *  \page signal_filtering_functions Filtering Functions
 *  @defgroup signal_filtering_functions Filtering Functions
 *  @ingroup npps
 * Functions that provide functionality of generating output signal
 * based on the input signal like signal integral, etc.
 *
 * @{
 *
 */

/** 
 * \section signal_integral Integral
 * @defgroup signal_integral Integral
 * Compute the indefinite integral of a given signal.
 * The i-th element is computed to be
 * \f[
 *      s'_i = \sum_0^i s_j
 * \f]
 *
 * @{
 */
 
#ifdef NPP_PLUS
namespace nppPlusV {
#endif
/**
 * IntegralGetBufferSize
 */
NPP_EXTERN_C
NppStatus
nppsIntegralGetBufferSize_32s(size_t nLength,  size_t * hpBufferSize);

/** 
 * Integral
 */ 
NPP_EXTERN_C
NppStatus 
nppsIntegral_32s_Ctx(const Npp32s * pSrc, Npp32s * pDst, size_t nLength, Npp8u * pDeviceBuffer, NppStreamContext nppStreamCtx);

#ifdef NPP_PLUS
}
#endif
/** @} signal_integral */

/** @} signal_filtering_functions */

#endif /* NV_NPPS_PLUS_FILTERING_FUNCTIONS_H */
